'use strict';

/**
 * Normalized the value according to the mapping table
 * @param {*} value The original value
 * @param {Object: Regexp|Function} table The mapping table
 */
function normalize(value, table) {
  for (var name in table) {
    if (typeof table[name] === 'function' && table[name](value)) return name;
    if (table[name] instanceof RegExp && table[name].test(value)) return name;
  }

  return value;
}

var createShortcut = (function (handles, env) {
  return Object.keys(handles).reduce(function (shortcut, name) {
    shortcut[name] = handles[name](env);
    return shortcut;
  }, {});
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

// import { bindShortcut, ufunctionpdateShortcut, clearShortcut } from './shortcut';
var create = (function () {
  var alias = {};
  var shortcutHandles = {};
  var originalEnv = null;

  var detector = function detector(policy) {
    return detector.detect(policy);
  };
  detector.value = null;

  detector.detect = function (policy) {
    if (!('default' in policy)) throw new TypeError('[detect-env detect()] expect deault value');

    var name = detector.value;
    return policy[name] || policy.default;
  };

  detector.alias = function (table) {
    for (var name in table) {
      if (!(table[name] instanceof RegExp || typeof table[name] === 'function')) {
        var message = '[detect-env alias()]: each alias.xxx should be an RegExp or Function, but get ' + _typeof(table[name]);
        throw new TypeError(message);
      }
    }
    alias = table;

    detector.value = normalize(originalEnv, alias);
    detector.is = createShortcut(shortcutHandles, detector.value);

    return detector;
  };

  detector.shortcut = function (handles) {
    for (var name in handles) {
      if (typeof handles[name] !== 'function') {
        var message = '[detect-env shortcut()]: each shortcut.xxx should be an Function, but get ' + _typeof(handles[name]);
        throw new TypeError(message);
      }
    }

    shortcutHandles = handles;
    detector.is = createShortcut(handles, detector.value);
    return detector;
  };

  detector.envVariable = function (variable) {
    if (variable !== null && (typeof variable === 'undefined' ? 'undefined' : _typeof(variable)) === 'object' || typeof variable === 'function') {
      throw new TypeError('[detect-env envVariable()] param should not be ' + (typeof variable === 'undefined' ? 'undefined' : _typeof(variable)));
    }

    originalEnv = variable;
    detector.value = normalize(variable, alias);
    detector.is = createShortcut(shortcutHandles, detector.value);

    return detector;
  };

  return detector;
});

var detector$1 = create().alias({
  prod: /^production$/,
  dev: /^develop(ment)?$/,
  stage: /^st$/,
  local: function local(envName) {
    return envName === undefined || envName === null || /(local)|(^$)/.test(envName);
  }
}).shortcut({
  prod: function prod(envName) {
    return envName === 'prod';
  },
  stage: function stage(envName) {
    return envName === 'stage';
  },
  dev: function dev(envName) {
    return envName === 'dev';
  },
  test: function test(envName) {
    return envName === 'test';
  },
  local: function local(envName) {
    return envName === 'loacal';
  }
}).envVariable(process ? process.env.NODE_ENV : '');

detector$1.create = create;

module.exports = detector$1;
